<?php

/**
* Creates the Kupindo catalog and taxanomy terms from the csv file
*/
function kupindo_add_categories(){
  $csv = array_map('str_getcsv', file(drupal_get_path('module', 'kupindo').'/kategorije.csv'));

  foreach ($csv as $key => $value) {
    $csv[$key] = explode(';', $value[0]);
  }
  $csv = _kupindo_sort_categories($csv);

  
  if(taxonomy_vocabulary_machine_name_load('kupindo_categories')){
    $kupindo_taxonomy = taxonomy_vocabulary_machine_name_load('kupindo_categories');
    taxonomy_vocabulary_delete($kupindo_taxonomy->vid);
    field_delete_field('field_kupindo_internal_id');
    field_purge_batch(1);
  }

  $kupindo_vocab = (object) array(
    'name' => 'Kupindo kategorije',
    'description' => 'Vocabulary for Kupindo categories',
    'machine_name' => 'kupindo_categories'
  );
  taxonomy_vocabulary_save($kupindo_vocab);

  $kupind_internal_id = array(
      'field_name' => 'field_kupindo_internal_id',
      'cardinality' => 1,
      'type' => 'number_integer',
  );
    field_create_field($kupind_internal_id);

    $kupind_internal_id_instance = array(
      'field_name' => 'field_kupindo_internal_id',
      'label' => 'Kupindo category ID',
      'bundle' => 'kupindo_categories',
      'entity_type' => 'taxonomy_term',
      'widget' => array(
        'type' => 'number',
      ),
    );
    field_create_instance($kupind_internal_id_instance);

  $taxonomy = taxonomy_vocabulary_machine_name_load('kupindo_categories');
  kupindo_create_taxonomy_terms($csv, $taxonomy->vid);
}

/**
* Loops through the CSV values and creates taxonomy terms
* @param array $csv - The array of values from the CSV
* @param int $vid - Catalog VID
* The script loops through the CSV array and saves the terms, and their parents accordingly
*/
function kupindo_create_taxonomy_terms($csv, $vid){
  foreach($csv as $key => $val){
    $kupindo_id = $val[0];
    $parent_kupindo_id = $val[1];
    $name = $val[2];
    $parent_name = isset($val[3]) ? $val[3] : $val[2];
    if(!isset($val[3])){
      $term = kupindo_create_term($name, '', $vid, $kupindo_id, $_SESSION[intval($val[1])]);
      taxonomy_term_save($term);
      continue;
    }
    if(strpos($parent_name, '/') !== FALSE){
      $parent_name_array = explode('/', $parent_name);
      unset($parent_name_array[sizeof($parent_name_array) -1]);
      $parent_parent_id = kupindo_create_parent($parent_name_array, $csv, $vid);
    }
    if(!taxonomy_get_term_by_name(trim($parent_name_array[sizeof($parent_name_array) - 1]))){
      $parent_term = kupindo_create_term($parent_name_array[sizeof($parent_name_array) - 1], '', $vid, $parent_kupindo_id, $parent_parent_id);
      taxonomy_term_save($parent_term);
      $parent_id = $parent_term->tid;
      $_SESSION[$parent_kupindo_id] = $parent_term->tid;
    }
    else{
      $parent_t = taxonomy_get_term_by_name(trim($parent_name_array[sizeof($parent_name_array) - 1]));
      $parent_t = reset($parent_t);
      $parent_id = $parent_t->tid;
      $_SESSION[$parent_kupindo_id] = $parent_id;
    }

    $term = (object) array(
      'name' => trim($name),
      'description' => '',
      'vid' => $vid,
    );
    $term->parent = $parent_id;
    $term->field_kupindo_internal_id['und'][0]['value'] = $kupindo_id;
    taxonomy_term_save($term);
    $_SESSION[$kupindo_id] = $term->tid;
  }
}

/**
* Creates all the parent categories of a line
* @param array $parent_array - Array of parent categories
* @param array $csv - Array of values taken from the CSV
* @param integer $vid - Catalog VID
* @return integer $parent_id - TID of the created taxonomy term
*/
function kupindo_create_parent($parent_array, $csv, $vid){
  $parent_id = 0;
  if(sizeof($parent_array) > 1){
    unset($parent_array[sizeof($parent_array) - 1]);
  }
  $parent_array = array_reverse($parent_array);
  foreach (array_reverse($parent_array) as $key => $value) {
    if(!taxonomy_get_term_by_name(trim($value))){
      $parent_id = kupindo_find_parent_in_csv($value, $csv, $vid, $parent_id);
    }
    else{
      $tax = taxonomy_get_term_by_name(trim($value));
      $parent_id = reset($tax)->tid;
    }
  }
  return $parent_id;
}

/**
* Looks for the parent Kupindo ID in the CSV array so it can be saved and used for child terms
* @param string $parent - Name of the parent
* @param array $csv - Array of values taken from CSV
* @param integer $vid - Catalog VID
* @param integer $parent_id - ID of the parent for current term
* @return integer $term->tid - TID of the new term
*/
function kupindo_find_parent_in_csv($parent, $csv, $vid, $parent_id){
  foreach ($csv as $key => $value) {
    if(!isset($value[3])){
      return $value[1];
    }
    if(strpos($value[3], '/') !== FALSE){
      $test = explode('/', $value[3]);
      unset($test[sizeof($test) - 1]);
      if(trim($test[sizeof($test) - 1]) == trim($parent)){
        $term = kupindo_create_term($parent, '', $vid, $value[1], $parent_id);
        taxonomy_term_save($term);
        $_SESSION[$value[1]] = $term->tid;
        return $term->tid;
      }
    }
  }
}

/**
* Creates the term object
* @param string $name - Name of the term
* @param string $description - Description of the term
* @param integer $vid - Catalog VID for the term
* @param integer $kupindo_id - Kupindo ID from the CSV
* @param integer $parent_id - TID of the parent term
* @return object $term
*/
function kupindo_create_term($name, $description, $vid, $kupindo_id, $parent_id = 0){
  $term = (object) array(
    'name' => trim($name),
    'description' => $description,
    'vid' => $vid
  );
  $term->field_kupindo_internal_id['und'][0]['value'] = $kupindo_id;
  $term->parent = $parent_id;
  return $term;
}

function kupindo_get_term_by_kupind_id($kupindo_id){
  $query = new EntityFieldQuery();
  $query->entityCondition('entity_type', 'taxonomy_term')
    // ->propertyCondition('vid', $vid)
    ->fieldCondition('field_kupindo_internal_id', 'value', $kupindo_id, '=');
  $result = $query->execute();
  print_r($result);
}

function _kupindo_sort_categories($csv){
  unset($csv[0]);
  usort($csv, function($a, $b){
    if(isset($a[3]) && isset($b[3])){
      return sizeof(explode('/', $a[3])) > sizeof(explode('/', $b[3]));
    }
    elseif(isset($a[3]) && !isset($b[3])){
      return true;
    }
    elseif(!isset($a[3]) && isset($b[3])){
      return false;
    }
    else{
      return true;
    }
  });
  foreach ($csv as $key => $value) {
    $new = array();
    if(sizeof($value) == 3){
      $new = $value;
      unset($csv[$key]);
      $csv[] = $new;
    }
  }
  return array_values($csv);
}