<?php

/**
* Creates the custom fields, their types and options
* @return array
*/
function returnCustomFields(){
  return array(
    array(
      'field_name' => 'kupindo_for_kupindo',
      'type' => 'list_boolean',
      'cardinality' => 1,
      'label' => 'Ide na Kupindo',
      'widget' => array(
          'type' => 'options_onoff',
          'settings' => array(
            'display_label' => TRUE
         )),
      'settings'    => array(
            'allowed_values' => array(
              'Ne',
              'Da',
            )),
      'description' => 'Označite ako proizvod treba da ide na Kupindo',
    ),
    array(
      'field_name' => 'kupindo_active_product',
      'type' => 'list_boolean',
      'cardinality' => 1,
      'label' => 'Proizvod aktivan',
      'widget' => array(
          'type' => 'options_onoff',
          'settings' => array(
            'display_label' => TRUE
         )),
      'settings'    => array(
            'allowed_values' => array(
              'Ne',
              'Da',
            )),
      'description' => 'Označite da li je proizvod aktivan na Kupindu',
    ),
    array(
      'field_name' => 'kupindo_subcategory_id',
      'type' => 'taxonomy_term_reference',
      'cardinality' => 1,
      'label' => 'Kupindo kategorija',
      'widget' => 'taxonomy_hs',
      'description' => 'Odaberite ID Kupindo kategorije za ovaj proizvod',
      'settings' => array(
        'allowed_values' => array(
          array(
            'vocabulary' => 'kupindo_categories',
            'parent' => 0
          ),
        )
      )
    ),
    array(
      'field_name' => 'kupindo_product_state',
      'type' => 'list_text',
      'cardinality' => 1,
      'label' => 'Stanje proizvoda',
      'widget' => 'options_select',
      'description' => 'Odaberite stanje proizvoda',
      'settings' => array(
        'allowed_values' => array(
          1 => 'Nekorišćen',
          2 => 'Nekorišćen sa felerom',
          3 => 'Polovan bez oštećenja',
          4 => 'Polovan sa vidljivim znacima korišćenja',
          5 => 'Neispravno'
        ),
      ),
    ),
    array(
      'field_name' => 'kupindo_shipping_type',
      'type' => 'list_text',
      'cardinality' => 8,
      'label' => 'Način slanja',
      'widget' => 'options_buttons',
      'description' => 'Odaberite način slanja proizvoda',
      'settings' => array(
        'allowed_values' => array(
          1 => 'AKS',
            2 => 'BEX',
            3 => 'City Express',
            4 => 'Pošta',
            5 => 'Post Express',
          6 => 'DExpress',
          7 => 'Lično preuzimanje',
          8 => 'Organizovani transport'
        )
      )
    ),
    array(
      'field_name' => 'kupindo_payment_type',
      'type' => 'list_text',
      'cardinality' => 5,
      'label' => 'Način plaćanja',
      'widget' => 'options_buttons',
      'description' => 'Odaberite način placanja prozivoda',
      'settings' => array(
        'allowed_values' => array(
          4 => 'Plaćanje pouzećem',
          5 => 'Lično',
          6 => 'Tekući račun (pre slanja)',
          7 => 'PostNet (pre slanja)',
          8 => 'Ostalo (pre slanja)'
        )
      )
    ),
    array(
      'field_name' => 'kupindo_free_shipping',
      'type' => 'list_boolean',
      'cardinality' => 1,
      'label' => 'Besplatna dostava',
      'widget' => array(
          'type' => 'options_onoff',
          'settings' => array(
            'display_label' => TRUE
         )),
      'settings'    => array(
            'allowed_values' => array(
              'Ne',
              'Da',
            )),
      'description' => 'Označite da li proizvod ima besplatnu poštarinu'
    ),
    array(
      'field_name' => 'kupindo_varanty',
      'type' => 'list_boolean',
      'cardinality' => 1,
      'label' => 'Garancija',
      'widget' => array(
          'type' => 'options_onoff',
          'settings' => array(
            'display_label' => TRUE
         )),
      'settings'    => array(
            'allowed_values' => array(
              'Ne',
              'Da',
            )),
      'description' => 'Označite da li proizvod ima garanciju'
    ),
    array(
      'field_name' => 'kupindo_varanty_length',
      'type' => 'text',
      'cardinality' => 1,
      'label' => 'Dužina garancije',
      'widget' => 'text_textfield',
      'description' => 'Upišite dužinu garancije (npr. Dva meseca, Šest meseci)'
    ),
  );
}

/**
* Creates custom fields for the commerce_product
*/
function kupindo_add_custom_fields(){
  $custom_fields = returnCustomFields();
  foreach($custom_fields as $key => $val){

    field_create_field($val);
    $product_types = commerce_product_types();
    foreach ($product_types as $type_key => $value) {
      if(!field_info_instance('commerce_product', $val['field_name'], $type_key)){
        $instance = array(
          'field_name' => $val['field_name'],
          'entity_type' => 'commerce_product',
          'label' => $val['label'],
          'bundle' => $type_key,
          'required' => FALSE,
          'settings' => array(),
          'widget' => array(
            'type' => $val['widget']
          )
        );
        $instance['widget'] = is_string($val['widget']) ? array('type' => $val['widget']) : $val['widget'];
        field_create_instance($instance);
      }
    }
  }
}

