<?php

/**
* Creates a new XML file with all the products that are tagged for Kupindo
*/
function kupindo_export_xml(){
  $query = db_query('SELECT entity_id FROM field_data_kupindo_for_kupindo')->fetchAll();
  
  $doc = new DOMDocument();
  $doc->formatOutput = TRUE;

  $root = $doc->createElement('Prodavnica');
  $root = $doc->appendChild($root);

  $product_list = $doc->createElement('PredmetiLista');
  $product_list = $root->appendChild($product_list);
  $product_list->setAttribute('type', 'array');

  foreach ($query as $key => $value) {
    $product = commerce_product_load($value->entity_id);
    // dsm();
    if(isset($product->kupindo_for_kupindo['und'][0]['value']) && $product->kupindo_for_kupindo['und'][0]['value'] == 1 && _kupindo_has_all_kupindo_fields($product)){
      $shipping = $product->kupindo_shipping_type['und'];
      $payment = $product->kupindo_payment_type['und'];
      if($msg = checkDeliveryTypes($shipping, $payment, $product->title)){
        dsm($msg, 'error');
        continue;
      }
      $child = $doc->createElement('Predmet');
      $child = $product_list->appendChild($child);

      _kupindo_create_dom_element($doc, $child, 'Aktivan', $product->kupindo_active_product['und'][0]['value']);

      $kupindo_category_term = taxonomy_term_load($product->kupindo_subcategory_id['und'][0]['tid']);

      _kupindo_create_dom_element($doc, $child, 'IDPodKategorija', $kupindo_category_term->field_kupindo_internal_id['und'][0]['value']);

      $product_state = isset($product->kupindo_product_state['und']) ? $product->kupindo_product_state['und'][0]['value'] : 1;


      checkPersonalDeliveryAndPayment($shipping, $payment);

      _kupindo_create_dom_element($doc, $child, 'Naziv', $product->title);
      _kupindo_create_dom_element($doc, $child, 'Opis', _kupindo_get_parent_description($value->entity_id));
      _kupindo_create_dom_element($doc, $child, 'Kolicina', format_product_stock($product->commerce_stock['und'][0]['value']));
      _kupindo_create_dom_element($doc, $child, 'Cena', $product->commerce_price['und'][0]['amount']);
      _kupindo_create_dom_element($doc, $child, 'Images', $product->field_image['und']);
      _kupindo_create_dom_element($doc, $child, 'IDInterni', $value->entity_id);
      _kupindo_create_dom_element($doc, $child, 'Stanje', $product_state);
      _kupindo_create_dom_element($doc, $child, 'NacinSlanja', $shipping);
      _kupindo_create_dom_element($doc, $child, 'NacinPlacanja', $payment);
      _kupindo_create_dom_element($doc, $child, 'BesplatnaDostava', $product->kupindo_free_shipping['und'][0]['value']);
      _kupindo_create_dom_element($doc, $child, 'Garancija', $product->kupindo_varanty['und'][0]['value']);
      _kupindo_create_dom_element($doc, $child, 'DuzinaGarancije', $product->kupindo_varanty_length['und'][0]['value']);
    }
  }
  drupal_set_message('XML kreiran. Možete ga preuzeti pomoću stavke iz menija, ili s sledećeg linka: <a href="admin/settings/kupindo/download-kupindo-xml">Preuzmi</a>');
  _kupindo_save_xml_from_string($doc);
  drupal_goto('<front>');
}

function checkDeliveryTypes(&$shipping, &$payment, $title, $value_type = 0){
  $shipping_agency = array(1,2,3,4,5,6);
  $payment_type = array(4,6);
  foreach ($shipping as $key => $value) {
      $val = $value_type == 0 ? $value['value'] : $value;
    if(in_array($val, $shipping_agency)){
      $payment_has_personal = FALSE;
      foreach ($payment as $p_key => $p_value) {
          $p_val = $value_type == 0 ? $p_value['value'] : $p_value;
        if(in_array($p_val, $payment_type)){
          $payment_has_personal = TRUE;
          break;
        }
      }
      if(!$payment_has_personal){
        return 'Ako ste odabrali slanje preko neke kurirske službe, morate odabrati i plaćanje pouzećem ili na tekući racun. Proizvod u pitanju: '.$title;
      }
    }
  }

  foreach ($payment as $key => $value) {
      $val = $value_type == 0 ? $value['value'] : $value;
    if(in_array($val, $payment_type)){
      $shipping_has_personal = FALSE;
      foreach ($shipping as $p_key => $p_value) {
          $p_val = $value_type == 0 ? $p_value['value'] : $p_value;
        if(in_array($p_val, $shipping_agency)){
          $shipping_has_personal = TRUE;
          break;
        }
      }
      if(!$shipping_has_personal){
        return 'Ako ste odabrali plaćanje pouzecem ili na tekući racun, morate odabrati neku kurirsku službu kao način slanja. Proizvod u pitanju: '.$title;
      }
    }
  }
  return false;
}

function checkPersonalDeliveryAndPayment(&$shipping, &$payment){
  foreach ($shipping as $key => $value) {
    if($value['value'] == 7){
      $payment_has_personal = FALSE;
      foreach ($payment as $p_key => $p_value) {
        if($p_value['value'] == 5){
          $payment_has_personal = TRUE;
          break;
        }
      }
      if(!$payment_has_personal){
        $payment[] = array('value' => '5');
        break;
      }
    }
  }
  foreach ($payment as $key => $value) {
    if($value['value'] == 5){
      $shipping_has_personal = FALSE;
      foreach ($shipping as $p_key => $p_value) {
        if($p_value['value'] == 7){
          $shipping_has_personal = TRUE;
          break;
        }
      }
      if(!$shipping_has_personal){
        $shipping[] = array('value' => '7');
        break;
      }
    }
  }
}

/**
* Checks if the products has all the Kupindo fields that are required
* @param object $product
* @return bool
*/
function _kupindo_has_all_kupindo_fields($product){
  $return =  isset($product->kupindo_subcategory_id['und'][0])
       && isset($product->kupindo_product_state['und'][0])
       && isset($product->kupindo_shipping_type['und'][0])
       && isset($product->kupindo_payment_type['und'][0])
       && isset($product->kupindo_free_shipping['und'][0])
       && isset($product->kupindo_varanty['und'][0]); 
  if(!$return){
    drupal_set_message('Proizvod: '.$product->title.' nema sva obavezna Kupindo polja i ne može uci u XML', 'error');
  }
  return $return;
}

/**
* Called from hook_entity_update(), this functions edits the XML depending on the fields in the product
* @param integer $id - Product ID
*/
function _kupindo_edit_xml($id){
  $product = commerce_product_load($id);
  $doc = _kupindo_get_xml_file();
  if(isset($product->kupindo_for_kupindo['und'][0])){
    if($product->kupindo_for_kupindo['und'][0]['value'] == 1 && _kupindo_has_all_kupindo_fields($product)){
      
      if(in_array($id, _kupindo_get_xml_internal_ids($doc))){
        _kupindo_edit_in_xml($doc, $id);
      }
      else{
        _kupindo_add_to_xml($doc, $id);
      }
    }
    else{
      if(in_array($id, _kupindo_get_xml_internal_ids($doc))){
        _kupindo_remove_from_xml($doc, $id);
      }
    }
  }
  else{
    if(in_array($id, _kupindo_get_xml_internal_ids($doc))){
      _kupindo_remove_from_xml($doc, $id);
    }
  }

  _kupindo_save_xml_from_string($doc);
}

/**
* Creates DOM elements in the XML
* @param object $dom - DOMDocument object
* @param object $product_list - Node of DOMDocument
* @param string $el_tag - Tag of the new element
* @param mixed $el_value - Value of the new element
*/
function _kupindo_create_dom_element(&$doc, &$product_list, $el_tag, $el_value){
  if(!is_array($el_value)){
    $child = $doc->createElement($el_tag, $el_value);
    $child = $product_list->appendChild($child);
  }
  else{
    switch ($el_tag) {
      case 'Images':
        _kupindo_create_dom_list($doc, $product_list, $el_tag, $el_value, 'ImageURL', true);
      break;

      case 'NacinSlanja':
        _kupindo_create_dom_list($doc, $product_list, $el_tag, $el_value, 'Slanje');
      break;

      case 'NacinPlacanja':
        _kupindo_create_dom_list($doc, $product_list, $el_tag, $el_value, 'Placanje');
      break;
    }
  }
}

/**
* Creates the child list for an element in the DOM
* @param object $dom - DOMDocument object
* @param object $product_list - Node of DOMDocument
* @param string $el_tag - Tag of the new element
* @param mixed $el_value - Value of the new element
* @param string $child_name - Tag name of the child element
* @param bool $image_url - True if the element to add is the image
*/
function _kupindo_create_dom_list(&$doc, &$product_list, $el_tag, $el_value, $child_name, $image_url = false){
  $parent = $doc->createElement($el_tag);
  $parent = $product_list->appendChild($parent);
  $parent->setAttribute('type', 'array');
  foreach ($el_value as $key => $value) {
    if($image_url){
      $child_value = str_replace('public:/', $_SERVER['HTTP_HOST'].'/sites/default/files', $value['uri']);
    }
    else{
      $child_value = $value['value'];
    }

    $child = $doc->createElement($child_name, $child_value);
    $child = $parent->appendChild($child);
  }
}

/**
* Saves the new XML file from the string
* @param object $doc - DOMDocument object
*/
function _kupindo_save_xml_from_string($doc){
  $str = $doc->saveXML($doc->documentElement);
  $kupindo_xml_file = fopen(getcwd() . "/sites/default/files/kupindo_products.xml",'w');
  fwrite($kupindo_xml_file, $str);
  fclose($kupindo_xml_file);
}

/**
* Gets all the product Id-s from the XML
* @param object $doc - DOMDocument object
* @return array $return - Array of ID-s
*/
function _kupindo_get_xml_internal_ids($doc){
  $return = array();
  for($i = 0; $i <= $doc->documentElement->getElementsByTagName('IDInterni')->length - 1; $i++){
    $return[] = $doc->documentElement->getElementsByTagName('IDInterni')->Item($i)->nodeValue;
  }
  return $return;
}

/**
* Removes the node based on the internal ID
* @param object $doc - DOMDocument object
* @param integer $id - Internal ID
*/
function _kupindo_remove_from_xml(&$doc, $id){
  foreach ($doc->documentElement->getElementsByTagName('Predmet') as $node) {
    if(intval($node->getElementsByTagName('IDInterni')->Item(0)->nodeValue) == intval($id)){
      $node->parentNode->removeChild($node);
    }
  }
  drupal_set_message('Proizvod '. commerce_product_load($id)->title .' uklonjen iz XML-a.');
}

/**
* Adds a new product to the XML
* @param object $doc - DOMDocument object
* @param integer $id - Product ID
*/
function _kupindo_add_to_xml(&$doc, $id){
  $product_list =  $doc->getElementsByTagName('PredmetiLista')->Item(0);

  $product = commerce_product_load($id);
  $shipping = $product->kupindo_shipping_type['und'];
  $payment = $product->kupindo_payment_type['und'];
  if($msg = checkDeliveryTypes($shipping, $payment, $product->title)){
    dsm($msg, 'error');
    return;
  }
  $child = $doc->createElement('Predmet');
  $child = $product_list->appendChild($child);

  _kupindo_create_dom_element($doc, $child, 'Aktivan', $product->kupindo_active_product['und'][0]['value']);

  $kupindo_category_term = taxonomy_term_load($product->kupindo_subcategory_id['und'][0]['tid']);
  _kupindo_create_dom_element($doc, $child, 'IDPodKategorija', $kupindo_category_term->field_kupindo_internal_id['und'][0]['value']);

  $product_state = isset($product->kupindo_product_state['und'][0]) ? $product->kupindo_product_state['und'][0]['value'] : 1;
  $kupindo_varanty_length = isset($product->kupindo_varanty_length['und'][0]) ? $product->kupindo_varanty_length['und'][0]['value'] : '';
  
  $shipping = $product->kupindo_shipping_type['und'];
  $payment = $product->kupindo_payment_type['und'];

  checkPersonalDeliveryAndPayment($shipping, $payment);
  _kupindo_create_dom_element($doc, $child, 'Naziv', $product->title);
  _kupindo_create_dom_element($doc, $child, 'Opis', _kupindo_get_parent_description($id));
  _kupindo_create_dom_element($doc, $child, 'Kolicina', format_product_stock($product->commerce_stock['und'][0]['value']));
  _kupindo_create_dom_element($doc, $child, 'Cena', $product->commerce_price['und'][0]['amount']);
  _kupindo_create_dom_element($doc, $child, 'Images', $product->original->field_image['und']);
  _kupindo_create_dom_element($doc, $child, 'IDInterni', $id);
  _kupindo_create_dom_element($doc, $child, 'Stanje', $product_state);
  _kupindo_create_dom_element($doc, $child, 'NacinSlanja', $shipping);
  _kupindo_create_dom_element($doc, $child, 'NacinPlacanja', $payment);
  _kupindo_create_dom_element($doc, $child, 'BesplatnaDostava', $product->kupindo_free_shipping['und'][0]['value']);
  _kupindo_create_dom_element($doc, $child, 'Garancija', $product->kupindo_varanty['und'][0]['value']);
  _kupindo_create_dom_element($doc, $child, 'DuzinaGarancije', $kupindo_varanty_length);
  drupal_set_message('Proizvod: '.$product->title.' dodan u XML.');
}

/**
* Edits an existing product in the XML
* @param object $doc - DOMDocument object
* @param integer $id - Product ID
*/
function _kupindo_edit_in_xml(&$doc, $id){
  foreach ($doc->documentElement->getElementsByTagName('Predmet') as $node) {
    if(intval($node->getElementsByTagName('IDInterni')->Item(0)->nodeValue) == intval($id)){
      $data = _kupindo_get_product_data_by_id($id);
      if(!empty($data)){
        if($msg = checkDeliveryTypes($shipping, $payment, $product->title, 1)){
          dsm($msg, 'error');
          return;
        }
        $data = _kupindo_get_product_data_by_id($id);
        $images_tag = $node->getElementsByTagName('ImageURL');
        $shipping_tag = $node->getElementsByTagName('Slanje');
        $payment_tag = $node->getElementsByTagName('Placanje');
        _kupindo_remove_tag_children($shipping_tag);
        _kupindo_remove_tag_children($payment_tag);
        _kupindo_remove_tag_children($images_tag);
        

        _kupindo_update_xml_node($doc, $node, $data['kupindo_active'], 'Aktivan');
        _kupindo_update_xml_node($doc, $node, $data['kupindo_category'], 'IDPodKategorija');
        _kupindo_update_xml_node($doc, $node, $data['title'], 'Naziv');
        _kupindo_update_xml_node($doc, $node, $data['description'], 'Opis');
        _kupindo_update_xml_node($doc, $node, $data['price'], 'Cena');
        _kupindo_update_xml_node($doc, $node, $data['stock'], 'Kolicina');
        _kupindo_update_xml_node($doc, $node, $data['images'], 'Images');
        _kupindo_update_xml_node($doc, $node, $data['internal_id'], 'IDInterni');
        _kupindo_update_xml_node($doc, $node, $data['kupindo_product_state'], 'Stanje');
        _kupindo_update_xml_node($doc, $node, $data['kupindo_shipping'], 'NacinSlanja');
        _kupindo_update_xml_node($doc, $node, $data['kupindo_payment'], 'NacinPlacanja');
        _kupindo_update_xml_node($doc, $node, $data['kupindo_free_shipping'], 'BesplatnaDostava');
        _kupindo_update_xml_node($doc, $node, $data['kupindo_varanty'], 'Garancija');
        _kupindo_update_xml_node($doc, $node, $data['kupindo_varanty_length'], 'DuzinaGarancije');
      }
    }
  }
  drupal_set_message('Proizvod '.$data['title']. ' je izmenjen u XML-u.');
}

/**
* Removes all the children from the parent tag
* @param object $tag - DOMDocument object of the tag
*/
function _kupindo_remove_tag_children(&$tag){
  if($tag->length != 0){
    while($tag->length){
      $tag->Item(0)->parentNode->removeChild($tag->Item(0));
    }
  }
}

/**
* Updates the values in the XML nodes
* @param object $doc - DOMDocument object
* @param object $node - DOMDocument node object
* @param mixed $node_value - New value for the node
* @param string $tag - Tag to be edited
*/
function _kupindo_update_xml_node(&$doc, &$node, $node_value, $tag){
  if(!is_array($node_value)){
    $old_value = $node->getElementsByTagName($tag)->Item(0);
    $new_value = $doc->createElement($tag, $node_value);
    $old_value->parentNode->replaceChild($new_value, $old_value);
  }
  else{
    switch ($tag) {
      case 'Images':
         _kupindo_create_child_in($doc, $node, $node_value, $tag, 'ImageURL', true);
      break;

      case 'NacinSlanja':
         _kupindo_create_child_in($doc, $node, $node_value, $tag, 'Slanje');
      break;

      case 'NacinPlacanja':
         _kupindo_create_child_in($doc, $node, $node_value, $tag, 'Placanje');
      break;
    }
  }
}

/**
* Creates the child list for an element in the DOM
* @param object $doc - DOMDocument object
* @param object $node - Node of DOMDocument
* @param array $node_value - Values of the new nodes
* @param string $parent_tag - Tag name of the parent
* @param string $child - Tag name of the child element
* @param bool $image_url - Check if the element in question is Image field
*/
function _kupindo_create_child_in(&$doc, &$node, $node_value, $parent_tag, $child, $image_url = false){
  $parent_tag_obj = $node->getElementsByTagName($parent_tag)->Item(0);
  foreach ($node_value as $key => $value) {
    if($image_url){
      $child_value = str_replace('public:/', $_SERVER['HTTP_HOST'].'/sites/default/files', $value['uri']);
    }
    else{
      $child_value = $value['value'];
    }
    $child_el = $doc->createElement($child, $child_value);
    $parent_tag_obj->appendChild($child_el);
  }
}

/**
* Gets the product description from the node
* @param integer $product_id - ID of the product
* @return string - Description of the product
*/
function _kupindo_get_parent_description($product_id){
  $result = db_query("SELECT entity_id FROM field_data_field_product WHERE entity_type = 'node' AND field_product_product_id=".$product_id)->fetchAll();
  $node = node_load($result[0]->entity_id);
  return strip_tags($node->body['und'][0]['safe_value']);
  dsm($node->body);
  return isset($node->body['und'][0]) ? trim(preg_replace('/ +/', ' ', preg_replace('/[^A-Za-z0-9 ]/', ' ', urldecode(html_entity_decode(strip_tags($node->body['und'][0]['value'])))))) : '';
}

/**
* Gets all the product data
* @param integer $id - Product ID
*/
function _kupindo_get_product_data_by_id($id){
  $product = commerce_product_load($id);

  $kupindo_category_term = taxonomy_term_load($product->kupindo_subcategory_id['und'][0]['tid']);
  $kupindo_category = $kupindo_category_term->field_kupindo_internal_id['und'][0]['value'];

  $description = _kupindo_get_parent_description($id);

  $images = $product->original->field_image['und'];

  $shipping = array();
  if(!empty($product->kupindo_shipping_type)){
    foreach ($product->kupindo_shipping_type['und'] as $key => $value) {
      $shipping[] = $value['value'];
    }
  }

  $payment = array();
  if(!empty($product->kupindo_payment_type)){
    foreach ($product->kupindo_payment_type['und'] as $key => $value) {
      $payment[] = $value['value'];
    }
  }

  if(!$product->kupindo_for_kupindo['und'][0]['value']){
    return array();
  }
  $product_state = isset($product->kupindo_product_state['und'][0]) ? $product->kupindo_product_state['und'][0]['value'] : 1;
  $varanty_length = isset($product->kupindo_varanty_length['und']) && !empty($product->kupindo_varanty_length['und']) ? $product->kupindo_varanty_length['und'][0]['value'] : '';
  
  if(in_array(7, $shipping) && !in_array(5, $payment)){
    $payment[] = '5';
  }
  if(in_array(5, $payment) && !in_array(7, $shipping)){
    $shipping[] = '7';
  }
  
  return array(
    'kupindo_active' => $product->kupindo_active_product['und'][0]['value'],
    'kupindo_category' => $kupindo_category,
    'title' => $product->title,
    'description' => $description,
    'stock' => format_product_stock($product->commerce_stock['und'][0]['value']),
    'price' => $product->commerce_price['und'][0]['amount'],
    'images' => $images,
    'internal_id' => $id,
    'kupindo_product_state' => $product_state,
    'kupindo_shipping' => $shipping,
    'kupindo_payment' => $payment,
    'kupindo_free_shipping' => $product->kupindo_free_shipping['und'][0]['value'],
    'kupindo_varanty' => $product->kupindo_varanty['und'][0]['value'],
    'kupindo_varanty_length' => $varanty_length
  );
}

function format_product_stock($stock){
    if(strpos($stock, '.') !== FALSE){
        $stock_num = explode('.', $stock)[0];
        if(strpos($stock_num, '-') !== FALSE){
            return 0;
        }
        else{
            return $stock_num;
        }
    }
    elseif(strpos($stock, '-') !== FALSE){
        return 0;
    }
    else{
        return $stock;
    }
}

/**
* Downloads the XML file
*/
function kupindo_download_xml(){
  $file = getcwd() . "/sites/default/files/kupindo_products.xml";
  header('Content-Type: application/download');
    header('Content-Disposition: attachment; filename="kupindo_products.xml"');
    header("Content-Length: " . filesize(getcwd() . "/sites/default/files/kupindo_products.xml"));

    $fp = fopen($file, "r");
    fpassthru($fp);
    fclose($fp);
}

/**
* Gets the content of the XML file for Kupindo
*/
function _kupindo_get_xml_file(){
  $doc=new DOMDocument();
  $doc->load(getcwd() . "/sites/default/files/kupindo_products.xml");
  return $doc;
}